# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module implements logic for persistence layer of osia

Currently the only supported persistance is via git.
Module is responsible for maintenance of the git repository
and to store the generated artifacts by the `openshift-install
binary."""
import logging
from git import Repo


def check_repository():
    """Function checks local repository if it is up2date with
    remote.
    In case when there is difference between upstream and
    local copy it tries to pull from remote.

    It returns the Repo object and remote associated with
    current tracking branch."""
    rep = Repo("./")
    remote = rep.active_branch.tracking_branch()
    fetches = rep.remotes[remote.remote_name].fetch()
    for fetch in fetches:
        if fetch.name == remote.name and fetch.commit != rep.commit():
            logging.warning("There are changes in remote repository, trying to pull")
            rep.remotes[remote.remote_name].pull()
    if rep.is_dirty():
        logging.warning("There are not committed changes in your repository, please fix this")

    return rep, remote


def write_changes(cluster_directory):
    """Function stages generated directory, which contains files
    generated by openshift-install function, creates commit,
    and pushes to the remote of tracking branch."""
    rep, remote = check_repository()
    rep.index.add(cluster_directory)
    logging.info("Commiting installer changes for cluster %s", cluster_directory)
    rep.index.commit(f"[OCP Installer] installation files for {cluster_directory} added")

    rep.remotes[remote.remote_name].push()


def delete_directory(cluster_directory):
    """Function deletes commited directory both
    from local copy and from the remote repository."""
    rep, remote = check_repository()
    logging.info("Removing cluster directory from git repository %s", cluster_directory)
    rep.index.remove(cluster_directory, working_tree=True, r=True, f=True)
    rep.index.commit(f"[OCP Installer] removed installation files for {cluster_directory}")
    rep.remotes[remote.remote_name].push()



"""Module implements utilitary functions shared by download
package"""
import logging
from pathlib import Path
from typing import Callable
from tempfile import NamedTemporaryFile

import requests


def get_data(tar_url: str,
             target: str,
             processor: Callable[[NamedTemporaryFile, str], Path]) -> str:
    """Function downloads file via http and runs it through
    processor function for extraction"""
    result = None
    logging.debug('[get_data] Starting the download of %s', tar_url)
    req = requests.get(tar_url, stream=True, allow_redirects=True)
    buf = NamedTemporaryFile()
    for block in req.iter_content(chunk_size=4096):
        buf.write(block)
    buf.flush()
    logging.debug('[get_data] Download finished, starting extraction')
    result = processor(buf, target)
    buf.close()

    logging.debug('[get_data] File extracted to %s', result.as_posix())
    return result.as_posix()



# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module implements common base for install-config
creation.
It also implements logic to obtain correct specification
for specified installation platform"""
from abc import abstractmethod, ABC
from typing import ClassVar
from jinja2 import Environment, PackageLoader


class AbstractInstaller(ABC):
    """Base object for configuration of install-config"""
    # pylint: disable=too-many-instance-attributes

    __env: Environment = None

    # pylint: disable=too-many-arguments
    def __init__(self,
                 cluster_name=None,
                 base_domain=None,
                 master_flavor=None,
                 master_replicas=None,
                 pull_secret_file=None,
                 ssh_key_file=None,
                 worker_flavor=None,
                 worker_replicas=None,
                 certificate_bundle_file=None,
                 cluster_directory=None,
                 skip_clean=False,
                 installer=None,
                 **unused_kwargs):
        self.cluster_name = cluster_name
        self.base_domain = base_domain
        self.master_flavor = master_flavor
        self.master_replicas = master_replicas
        self.pull_secret_file = pull_secret_file
        self.pull_secret = None
        self.ssh_key_file = ssh_key_file
        self.ssh_key = None
        self.certificate_bundle_file = certificate_bundle_file
        self.certificate_bundle = None
        self.worker_flavor = worker_flavor
        self.worker_replicas = worker_replicas
        self.cluster_directory = cluster_directory
        self.skip_clean = skip_clean
        self.installer = installer

    @abstractmethod
    def acquire_resources(self):
        """Method which is called before the install-config can be generated.
        The method should be used to get all necessary dependencies to fill
        in details in install-config"""

    @abstractmethod
    def get_template_name(self):
        """Method to obtain name of jinja template related to specified platform."""

    @abstractmethod
    def post_installation(self):
        """Method called after the installation is done, this is the place where,
        things like registration of apps domain and other is expected to happen."""

    def check_clean(self):
        """Method returns if installation is configured to clean resources
        on failure."""
        return not self.skip_clean

    def process_template(self):
        """Method executes creation of install-config.yaml"""
        with open(self.pull_secret_file) as ps_file:
            self.pull_secret = ps_file.read()
        with open(self.ssh_key_file) as key_file:
            self.ssh_key = key_file.read()
        if self.certificate_bundle_file is not None:
            with open(self.certificate_bundle_file) as cert_file:
                self.certificate_bundle = cert_file.read()
        template = AbstractInstaller.get_environment().get_template(self.get_template_name())
        result = template.render(self.__dict__)
        with open(f"{self.cluster_name}/install-config.yaml", "w") as yaml_file:
            yaml_file.write(result)

    @classmethod
    def get_environment(cls) -> Environment:
        """Method loads jinja templates"""
        if cls.__env is None:
            cls.__env = Environment(loader=PackageLoader("osia.installer"))
        return cls.__env


class InstallerProvider:
    """Class implements dynamic provider of registered platform specific
    implementations of AbstractInstaller.
    Class implements singleton design pattern."""
    __instance: "InstallerProvider" = None

    @classmethod
    def instance(cls) -> "InstallerProvider":
        """Method to obtains singleton instance."""
        if cls.__instance is None:
            cls.__instance = cls()
        return cls.__instance

    @classmethod
    def register(cls, name, instance):
        """Method to dynamically register implementation of AbstractInstaller"""
        cls.instance().add_installer(name, instance)

    def __init__(self):
        self.installers = {}

    def add_installer(self, name: str, instance: ClassVar):
        """Insert concrete implementation of AbstractInstaller into the registry"""
        self.installers[name] = instance

    def __getitem__(self, name: str) -> ClassVar:
        return self.installers[name]



"""Module implements logic for rhcos image download"""
from subprocess import Popen, PIPE
from shutil import copyfileobj
from pathlib import Path
from typing import Tuple
from tempfile import NamedTemporaryFile

import gzip
import re
import logging
import json

import requests
from .utils import get_data

GITHUB_URL = "https://raw.githubusercontent.com/openshift/installer/{commit}/data/data/rhcos.json"


def get_commit(installer: str) -> str:
    """Function extracts source commit from installer,
    in order to find associated rhcos image"""
    version_str = ""
    commit_regex = re.compile(r"^.*commit (?P<commit>\w*)$", re.MULTILINE)
    with Popen([installer, "version"], stdout=PIPE, universal_newlines=True) as proc:
        version_str = proc.stdout.read()
    commits = commit_regex.findall(version_str)
    logging.info("Found commits by running installer %s", commits)
    return commits[0]


def get_url(installer: str) -> Tuple[str, str]:
    """Function builds url to rhcos image and version of
    rhcos iamge."""
    commit = get_commit(installer)
    gh_data_link = GITHUB_URL.format(commit=commit)
    rhcos_json = requests.get(gh_data_link, allow_redirects=True)
    rhcos_data = json.loads(rhcos_json.content)
    return rhcos_data['baseURI'] + rhcos_data['images']['openstack']['path'], rhcos_data['buildid']


def _extract_gzip(buff: NamedTemporaryFile, target: str) -> Path:
    result = None
    with gzip.open(buff.name) as zip_file:
        result = Path(target)
        with result.open("wb") as output:
            copyfileobj(zip_file, output)
    return result


def download_image(image_url: str, image_file: str):
    """Main entrypoint for image download, function
    extracts url to rhcos image, downloads and extracts it
    to specified target"""
    res_file = get_data(image_url, image_file, _extract_gzip)
    return res_file



# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module contains basics and common functionality to set up DNS."""
from abc import ABC, abstractmethod
from typing import ClassVar, Optional
from os import path
from pathlib import Path
import json


class DNSProvider:
    """Class implements dynamic provier of DNSUtil base class"""
    __instance = None

    # pylint: disable=protected-access
    @classmethod
    def register_provider(cls, name: str, clazz: ClassVar):
        """Method to dynamically register new implementation of
        DNSUtil"""
        cls.instance().__add_provider(name, clazz)

    @classmethod
    def instance(cls) -> "DNSProvider":
        """Returns singleton instance"""
        if cls.__instance is None:
            cls.__instance = cls()
        return cls.__instance

    def __init__(self):
        self.providers = dict()

    def __add_provider(self, name: str, clazz: ClassVar):
        self.providers[name] = clazz

    def __getitem__(self, name: str) -> ClassVar:
        return self.providers[name]

    def load(self, directory: str) -> Optional['DNSUtil']:
        """Method loads saved configuration of specific DNSUtil from
        file"""
        for k in self.providers:
            if path.exists(f"{directory}/{k}.json"):
                res = self[k]()
                res.unmarshall(directory)
                return res
        return None


class DNSUtil(ABC):
    """Class implements basic settings for """
    def __init__(self,
                 cluster_name=None,
                 base_domain=None,
                 ttl=None):
        self.cluster_name = cluster_name
        self.base_domain = base_domain
        self.ttl = ttl

    @abstractmethod
    def add_api_domain(self, ip_addr: str):
        """Method registers api domain in selected provider"""

    @abstractmethod
    def add_apps_domain(self, ip_addr: str):
        """Method registers apps domain in selected provider"""

    @abstractmethod
    def delete_domains(self):
        """Method deletes all registered domains in provider"""

    @abstractmethod
    def provider_name(self):
        """Get name of provider"""

    def marshall(self, out_dir: str):
        """Method stores current configuration on DNS provider to $provider_name.json"""
        with open(f"{out_dir}/{self.provider_name()}.json", "w") as output:
            json.dump(self, output, default=lambda o: o.__dict__)

    def unmarshall(self, in_dir: str):
        """Method loads stored configuration of DNS into provider object"""
        with open(f"{in_dir}/{self.provider_name()}.json") as in_stream:
            dns_conf = json.load(in_stream)
            for k in dns_conf:
                setattr(self, k, dns_conf[k])

    def delete_file(self):
        """Method removes unneeded configuration file"""
        file_path = Path(self.cluster_name) / f"{self.provider_name()}.json"
        if file_path.exists():
            file_path.unlink()



# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module implements dns methods to work with route53 provider"""
import boto3
from osia.installer.dns.base import DNSUtil


def _get_connection():
    return boto3.client('route53')


class Route53Provider(DNSUtil):
    """Class implements DNSUtil base specific for route53"""
    def __init__(self, api_ip=None, apps_ip=None, **kwargs):
        super().__init__(**kwargs)

        self.zone_id = None
        self.api_ip = api_ip
        self.apps_ip = apps_ip

    def provider_name(self):
        return 'route53'

    def _get_hosted_zone(self):
        if self.zone_id is None:
            zones = _get_connection().list_hosted_zones()['HostedZones']
            result = [v['Id'] for v in zones if v['Name'] == (self.base_domain + ".")]
            if len(result) == 0:
                raise Exception("Invalid number of results")
            self.zone_id = result[0]
        return self.zone_id

    def _execute_command(self, prefix: str, mode: str, ip_addr: str):
        change_batch = {
            'Changes': [
                {'Action': mode,
                 'ResourceRecordSet': {
                     'Name': '.'.join([prefix, self.cluster_name, self.base_domain]) + '.',
                     'Type': 'A',
                     'TTL': self.ttl,
                     'ResourceRecords': [
                         {'Value': ip_addr}
                     ]
                 }
                 }
            ]
        }
        _get_connection().change_resource_record_sets(
            HostedZoneId=self._get_hosted_zone(),
            ChangeBatch=change_batch)

    def add_api_domain(self, ip_addr: str):
        self.api_ip = ip_addr
        self._execute_command('api', 'CREATE', ip_addr)

    def add_apps_domain(self, ip_addr: str):
        self.apps_ip = ip_addr
        self._execute_command('*.apps', 'CREATE', ip_addr)

    def delete_domains(self):
        if self.api_ip is not None:
            self._execute_command('api', 'DELETE', self.api_ip)
        if self.apps_ip is not None:
            self._execute_command('*.apps', 'DELETE', self.apps_ip)
        self.delete_file()



# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Executor module implements controlling logic of cluster installation"""
from os import environ
from subprocess import Popen
from pathlib import Path
import logging

from .clouds import InstallerProvider
from .clouds.openstack import delete_fips, delete_image
from .dns import DNSProvider


class InstallerExecutionException(Exception):
    """Class represents exception raised by installer failure"""
    def __init__(self, *args, **kwargs):
        super().__init__(self, *args, **kwargs)


def execute_installer(installer, base_path, operation, os_image=None):
    """Function executes actual installation of OpenShift"""
    additional_env = None
    if os_image is not None and os_image:
        additional_env = environ.copy()
        additional_env.update({'OPENSHIFT_INSTALL_OS_IMAGE_OVERRIDE': os_image})
    with Popen([installer, operation, 'cluster', '--dir', base_path],
               env=additional_env, universal_newlines=True) as proc:
        proc.wait()
        if proc.returncode != 0:
            raise InstallerExecutionException("Failed execution of installer")


def install_cluster(cloud_provider,
                    cluster_name, configuration,
                    installer,
                    dns_settings=None):
    """Function represents main entrypoint to all logic necessary for
    cluster's deployment."""
    # pylint: disable=too-many-arguments
    cluster_path = Path("./") / cluster_name
    if cluster_path.exists():
        logging.error("Path %s already exists, remove it before continuing",
                      cluster_path.as_posix())
        return
    cluster_path.mkdir()
    inst = InstallerProvider.instance()[cloud_provider](cluster_name=cluster_name, **configuration)
    inst.acquire_resources()
    dns_prov = None
    if dns_settings is not None:
        dns_prov = DNSProvider.instance()[dns_settings['provider']](**dns_settings['conf'])
        dns_prov.add_api_domain(inst.osp_fip)
        dns_prov.marshall(cluster_name)

    inst.process_template()

    try:
        execute_installer(installer, cluster_name, 'create',
                          os_image=getattr(inst, 'os_image', None))
    except InstallerExecutionException as exception:
        logging.error(exception)
        if inst.check_clean():
            delete_cluster(cluster_name, installer)
            return

    inst.post_installation()

    if dns_settings is not None:
        dns_prov.add_apps_domain(inst.apps_fip)
        dns_prov.marshall(cluster_name)


def delete_cluster(cluster_name, installer):
    """Function is the controller of all actions leading to the
    cluster's deletion."""
    dns_prov = DNSProvider.instance().load(cluster_name)
    if dns_prov is not None:
        dns_prov.delete_domains()
    fips_file = Path(cluster_name) / "fips.json"
    if fips_file.exists():
        delete_fips(fips_file)
        delete_image(fips_file, cluster_name)
        fips_file.unlink()
    for k in [1, 2]:
        try:
            logging.debug("Attempt to clean #%d", k)
            execute_installer(installer, cluster_name, 'destroy')
            break
        except InstallerExecutionException as exception:
            logging.error("Re-executing installer due to error %s", exception)



# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module implements command line interface for the installation of
openshift"""
import argparse
import logging
from typing import List
import coloredlogs

from dynaconf import settings
from .installer import install_cluster, delete_cluster, storage, download_installer


def _identity(in_attr: str) -> str:
    return in_attr


def _read_list(in_str: str) -> List[str]:
    return in_str.split(',')


ARGUMENTS = {
    'common': {
        'cloud': {'help': 'Cloud provider to be used.', 'type': str,
                  'choices': ['openstack', 'aws']},
        'dns_provider': {'help': 'Provider of dns used with openstack cloud',
                         'type': str, 'choices': ['nsupdate', 'route53']}
    },
    'install': {
        'os_image': {'help': 'Image to override', 'type': str},
        'base_domain': {'help': 'Base domain for the cluster', 'type': str},
        'master_flavor': {'help': 'Flavor used for master nodes'},
        'master_replicas': {'help': 'Number of replicas for master nodes', 'type': int},
        'pull_secret_file': {'help': 'File containing pull secret from `cloud.redhat.com`'},
        'ssh_key_file': {'help': 'File with ssh key used by installer'},
        'list_of_regions': {'help': 'List of regions, comma separated values', 'proc': _read_list},
        'osp_cloud': {'help': 'Name of openstack cloud identity in clouds.yaml'},
        'osp_base_flavor': {'help': 'Base flavor to be used in openstack'},
        'osp_image_download': {'help': 'Enable image download by osia instead of openshift-install',
                               'action': 'store_true'},
        'osp_image_unique': {'help': 'Upload unique image per cluster',
                             'action': 'store_true'},
        'network_list': {'help': 'List of usable openstack networks, comma separated values',
                         'proc': _read_list},
        'worker_flavor': {'help': 'flavor of worker node'},
        'worker_replicas': {'help': 'Number of replicas of worker nodes', 'type': int},
        'certificate_bundle_file': {'help': 'CA bundle file'},
        'images_dir': {'help': 'Directory where images should be stored', 'type': str,
                       'default': 'images'},
        'skip_clean': {'help': 'Skip clean when installation fails', 'action': 'store_true'},
    },
    'deprecated': {
        'psi_cloud': {'help': 'DEPRECATED see osp_cloud'},
        'psi_base_flavor': {'help': 'DEPRECATED see osp_base_flavor'},
    },
    'dns': {
        'dns_ttl': {'help': 'TTL of the records', 'type': int},
        'dns_key_file': {'help': 'Keyfile used to access dns server via nsupdate'},
        'dns_zone': {'help': 'Zone on server where the record will be stored'},
        'dns_server': {'help': 'Address of server with running bind'}
    }
}

for a in [v for a in ARGUMENTS for u, v in ARGUMENTS[a].items()]:
    if not a.get('proc', None):
        a['proc'] = _identity


def _resolve_installer(from_args):
    if from_args.installer is None and from_args.installer_version is None:
        raise Exception('Either installer or installer-version must be passed')
    if from_args.installer:
        return from_args.installer

    return download_installer(from_args.installer_version,
                              from_args.installers_dir,
                              from_args.installer_source)


def _merge_dictionaries(from_args):
    result = {'cloud': None,
              'dns': None,
              'installer': _resolve_installer(from_args),
              'cloud_name': None,
              'cluster_name': from_args.cluster_name}
    if not from_args.__contains__('cloud'):
        return result
    defaults = settings.as_dict()
    if from_args.dns_provider is not None:
        result['dns'] = {'provider': from_args.dns_provider,
                         'conf': defaults['DNS'][from_args.dns_provider]}
        result['dns']['conf'].update(
            {j[4:]: i['proc'](vars(from_args)[j])
             for j, i in ARGUMENTS['dns'].items()
             if vars(from_args)[j] is not None}
        )
    if from_args.cloud is not None:
        result['cloud_name'] = from_args.cloud
        result['cloud'] = defaults['CLOUD'][from_args.cloud]
        result['cloud'].update(
            {j: i['proc'](vars(from_args)[j]) for j, i in ARGUMENTS['install'].items()
             if vars(from_args)[j] is not None}
        )
        # pylint: disable=unsupported-assignment-operation
        result['cloud']['installer'] = result['installer']
        if result['dns'] is not None:
            result['dns']['conf'].update({
                'cluster_name': from_args.cluster_name,
                'base_domain': result['cloud']['base_domain']
            })
    return result


def _exec_install_cluster(args):
    conf = _merge_dictionaries(args)
    if not args.skip_git:
        storage.check_repository()
    logging.info('Starting the installer with cloud name %s', conf['cloud_name'])
    install_cluster(
        conf['cloud_name'],
        conf['cluster_name'],
        conf['cloud'],
        conf['installer'],
        dns_settings=conf['dns']
    )
    if not args.skip_git:
        storage.write_changes(conf['cluster_name'])


def _exec_delete_cluster(args):
    conf = _merge_dictionaries(args)

    if not args.skip_git:
        storage.check_repository()

    delete_cluster(conf['cluster_name'], conf['installer'])

    if not args.skip_git:
        storage.delete_directory(conf['cluster_name'])


def _get_helper(parser: argparse.ArgumentParser):
    def printer(unused_conf):
        print("Operation not set, please specify either install or clean!")
        parser.print_help()
    return printer


def _create_commons():
    commons = argparse.ArgumentParser(add_help=False)
    common_arguments = [
        [['--cluster-name'], dict(required=True, help='Name of the cluster')],
        [['--installer'], dict(required=False,
                               help='Executable binary of openshift install cli', default=None)],
        [['--installer-version'], dict(help='Version of downloader to be downloaded',
                                       default='latest', type=str)],
        [['--installer-source'], dict(type=str,
                                      help='Set the source to search for installer',
                                      choices=["prod", "devel", "prev"],
                                      default='prod')],
        [['--installers-dir'], dict(help='Folder where installers are stored',
                                    required=False, default='installers')],
        [['--skip-git'], dict(help='When set, the persistance will be skipped',
                              action='store_true')],
        [['-v', '--verbose'], dict(help='Increase verbosity level', action='store_true')]
    ]
    for k in common_arguments:
        commons.add_argument(*k[0], **k[1])
    return commons


def _setup_parser():
    commons = _create_commons()

    parser = argparse.ArgumentParser("osia")
    parser.set_defaults(func=_get_helper(parser))
    sub_parsers = parser.add_subparsers()

    install = sub_parsers.add_parser('install', help='Install new cluster', parents=[commons])

    for arg, value in sorted({k: v for a in ARGUMENTS for k, v in ARGUMENTS[a].items()}.items()):
        install.add_argument(f"--{arg.replace('_', '-')}",
                             **{k: v for k, v in value.items() if k != 'proc'})
    install.set_defaults(func=_exec_install_cluster)

    clean = sub_parsers.add_parser('clean', help='Remove cluster', parents=[commons])
    clean.set_defaults(func=_exec_delete_cluster)
    return parser


def main_cli():
    """Function represents main entrypoint for the
    osia intaller

    It sets up the cli and starts the executor."""
    parser = _setup_parser()
    args = parser.parse_args()
    if vars(args).get('verbose', False):
        coloredlogs.install(level='DEBUG')
    else:
        coloredlogs.install(level='INFO')
    logging.captureWarnings(True)

    args.func(args)



#!/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2020 Osia authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
Modified by Madoshakalaka@Github (dependency links added)
"""

from setuptools import setup, find_packages
from os import path
from io import open


here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="osia",
    version="0.2.0-alpha2",
    description="OpenShift infra automation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://redhat-cop.github.io/osia/",
    author="Miroslav Jaros",
    author_email="mirek@mijaros.cz",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords="OpenShift OpenStack AWS automation",  # Optional
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    package_data={"": ["*.jinja2"]},
    entry_points={"console_scripts": ["osia = osia.cli:main_cli"]},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        "appdirs==1.4.4",
        "beautifulsoup4==4.9.3",
        "boto3==1.16.50",
        "botocore==1.19.50",
        "certifi==2020.12.5",
        "cffi==1.14.4",
        "chardet==4.0.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        "coloredlogs==15.0",
        "cryptography==3.3.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4, 3.5'",
        "decorator==4.4.2",
        "dogpile.cache==1.1.1; python_version >= '3.6'",
        "dynaconf[yaml]==3.1.2",
        "gitdb==4.0.5; python_version >= '3.4'",
        "gitpython==3.1.12",
        "humanfriendly==9.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        "idna==2.10; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "iso8601==0.1.13",
        "jinja2==2.11.2",
        "jmespath==0.10.0; python_version >= '2.6' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "jsonpatch==1.28; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        "jsonpointer==2.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "keystoneauth1==4.3.0; python_version >= '3.6'",
        "markupsafe==1.1.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "munch==2.5.0",
        "netifaces==0.10.9",
        "openstacksdk==0.52.0",
        "os-service-types==1.7.0",
        "pbr==5.5.1; python_version >= '2.6'",
        "pycparser==2.20; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "python-dateutil==2.8.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "pyyaml==5.3.1",
        "requests==2.25.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        "requestsexceptions==1.4.0",
        "ruamel.yaml==0.16.12",
        "s3transfer==0.3.3",
        "six==1.15.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "smmap==3.0.4; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "soupsieve==2.1; python_version >= '3.0'",
        "stevedore==3.3.0; python_version >= '3.6'",
        "urllib3==1.26.2; python_version != '3.4'",
    ],
    extras_require={"dev": []},
    dependency_links=[],
    project_urls={
        "Bug Reports": "https://github.com/redhat-cop/osia/issues",
        "Source": "https://github.com/redhat-cop/osia",
    },
)
